var game = new Phaser.Game(400, 490, Phaser.AUTO, "gameDiv");

var menuState = {};
menuState.preload = function() {
  game.stage.backgroundColor = "#ffc0c0";
  game.load.image("start_button", "assets/start_button.png");
  game.load.image("start_button_down", "assets/start_button_down.png");
};

menuState.create = function() {
  this.startBtn = game.add.button(0, 0, "start_button", this.gotoMain, this,
                                  "start_button_down");
  //this.startBtn.scale = 2;
};

menuState.update = function() {
};

menuState.gotoMain = function() {
  // Move game to mainState...
  game.state.start("main");
};


var mainState = {
  preload: function() {
    game.stage.backgroundColor = "#71c5cf";
    game.load.image("bird", "assets/earth.png");
    game.load.image("pipe", "assets/pipe.png");
  },

  create: function() {
    game.physics.startSystem(Phaser.Physics.ARCADE);
    this.bird = this.game.add.sprite(100, 245, "bird");
    game.physics.arcade.enable(this.bird);
    this.bird.body.gravity.y = 1000;
    var spaceKey = this.game.input.keyboard.addKey(Phaser.Keyboard.SPACEBAR);
    spaceKey.onDown.add(this.jump, this);

    this.pipes = game.add.group();
    this.pipes.enableBody = true;
    this.pipes.createMultiple(20, "pipe");

    this.timer = game.time.events.loop(1500, this.addRowOfPipes, this);

    this.score = 0;
    this.labelScore = game.add.text(20, 20, "0",
                                    { font: "30px Arial", fill: "#ffffff" });
  },

  update: function() {
    if (!this.bird.inWorld) {
      this.restartGame();
    }

    game.physics.arcade.overlap(this.bird, this.pipes, this.restartGame, null, this);
  },
  
  //-----------
  addOnePipe: function(x, y) {
    var pipe = this.pipes.getFirstDead();
    pipe.reset(x, y);
    pipe.body.velocity.x = -200;
    pipe.checkWorldBounds = true;
    pipe.outOfBoundsKill = true;
  },

  addRowOfPipes: function() {
    var hole = Math.floor(Math.random() * 5) + 1;
    for (var i = 0; i < 8; ++i) {
      if (i != hole && i != hole + 1) {
        this.addOnePipe(400, i * 60 + 10);
      }
    }
    ++this.score;
    this.labelScore.text = this.score;
  },
  
  jump: function() {
    this.bird.body.velocity.y = -350;
  },

  restartGame: function() {
    game.state.start("menu");
  },
};

game.state.add("menu", menuState);
game.state.add("main", mainState);
game.state.start("menu");
