var game = new Phaser.Game(800, 600, Phaser.AUTO, "gameDiv");

var menuState = {};
menuState.preload = function() {
  game.stage.backgroundColor = "#303030";
  game.load.image("menu", "assets/menu.png");
  game.load.image("stars", "assets/stars.png");
  game.load.spritesheet("start", "assets/start.png", 116, 35);
  
  game.load.audio("menutune", ["assets/menutune.ogg", "assets/menutune.mp3"], true);
};

menuState.create = function() {
  this.back = game.add.sprite(0, 0, "menu");
  this.back.smoothed = false;
  this.back.scale.set(4);
  this.stars = game.add.sprite(0, 0, "stars");
  this.stars2 = game.add.sprite(800, 0, "stars");
  this.menutune = game.add.audio("menutune", 0.7, true, true);
  this.menutune.play('', 0, 0.4, true);

  this.button = game.add.button(168, 368, "start", menuState.gotoMain, this,
                                0, 1, 0, 1);
  this.button.smoothed = false;
  this.button.scale.set(4);
};

menuState.update = function() {
  this.stars.x -= 1;
  this.stars2.x -= 1;
  if (this.stars.x + 800 < 0) {
    this.stars.x += 800;
    this.stars2.x += 800;
  }
};

menuState.gotoMain = function() {
  this.menutune.stop();
  // Move game to mainState...
  game.state.start("main");
};


var MoveDef = function(src, dest, spritename) {
  this.src = src;
  this.dest = dest;
  this.spritename = spritename;
  this.sprite = game.add.sprite(this.src.x - this.src.deltaCursor + 76,
                                this.src.y - this.src.deltaCursor + 76, spritename);
  this.sprite.smoothed = false;0
  this.sprite.scale.set(3);
  this.sprite.animations.add("main", null, 10, true);
  this.sprite.animations.play("main");
};

var NORMAL_FRAME = 0;
var FROZEN_FRAME = 1;
var BOILING_FRAME = 2;

var numToStr = function(num) {
  var str = "" + num;
  if (str.indexOf(".") >= 0) {
    str = str.substr(0, str.indexOf(".") + 2);
  }
  return str;
};

var mainState = {
  moves: [],
  
  preload: function() {
    game.stage.backgroundColor = "#202020";
    game.load.spritesheet("earth", "assets/earth.png", 50, 50);
    game.load.spritesheet("fire", "assets/fire.png", 32, 32);
    game.load.spritesheet("air", "assets/air.png", 45, 45);
    game.load.spritesheet("water", "assets/water.png", 40, 40);
    game.load.image("textarea", "assets/textarea.png");
    game.load.image("selection", "assets/selection.png");
    game.load.image("cursor", "assets/cursor.png");
    game.load.bitmapFont("nokiafont","assets/nokia.png", "assets/nokia.xml");
    game.load.spritesheet("fireelem", "assets/fireelem.png", 8, 8);
    game.load.spritesheet("buttons", "assets/buttons.png", 16, 16);
  },

  create: function() {
    this.stars = game.add.sprite(0, 0, "stars");
    this.stars2 = game.add.sprite(800, 0, "stars");
    
    this.earth = game.add.sprite(0, 0, "earth");
    this.earth.deltaCursor = 0;
    this.earth.model = EarthWorld;
    this.fire = game.add.sprite(0, 0, "fire");
    this.fire.deltaCursor = 35;
    this.fire.model = FireWorld;
    this.air = game.add.sprite(0, 0, "air");
    this.air.deltaCursor = 10;
    this.air.model = AirWorld;
    this.water = game.add.sprite(0, 0, "water");
    this.water.deltaCursor = 20;
    this.water.model = WaterWorld;
    this.planets = [ this.earth, this.fire, this.air, this.water ];
    
    for (var p in this.planets) {
      var planet = this.planets[p];
      planet.smoothed = false;
      planet.scale.set(4);
      planet.inputEnabled = true;
    }
    this.selected = null;
    this.pointed = null;

    this.selection = game.add.sprite(-1000, -1000, "selection");
    this.selection.smoothed = false;
    this.selection.scale.set(4);
    this.cursor = game.add.sprite(-1000, -1000, "cursor");
    this.cursor.smoothed = false;
    this.cursor.scale.set(4);
    
    this.angle = 0;

    this.texts = game.add.group();
    this.textarea = game.add.sprite(600, 0, "textarea");
    this.textarea.smoothed = false;
    this.textarea.scale.set(4);
    this.textarea.inputEnabled = true;
    this.textgroup = game.add.group();
    this.textnum = 0;
  },

  addMove: function(src, dest, spritename) {
    this.moves.push(new MoveDef(src, dest, spritename));
  },

  removeMove: function(src, dest, spritename) {
    for (var i = this.moves.length - 1; i >= 0; --i) {
      var move = this.moves[i];
      if ((move.src === src) && (move.dest === dest) && (move.spritename === spritename)) {
        move.sprite.destroy();
        this.moves.splice(i, 1);
        return;
      }
    }
  },

  ROUND_RATE: 10,

  lastRound: 0,

  update: function() {
    this.lastRound = (this.lastRound + 1) % this.ROUND_RATE;
    if (this.lastRound == 0) {
      U.doRound();
    }
    
    this.stars.x -= 0.1;
    this.stars2.x -= 0.1;
    if (this.stars.x + 800 < 0) {
      this.stars.x += 800;
      this.stars2.x += 800;
    }
    
    this.angle += 0.001;
    if (this.angle > Math.PI) {
      this.angle -= 2 * Math.PI;
    }
    this.earth.x = 200 - 180 * Math.cos(this.angle);
    this.earth.y = 200 - 170 * Math.sin(this.angle);
    this.fire.x = 225 - 220 * Math.cos(this.angle + 0.55 * Math.PI);
    this.fire.y = 235 - 230 * Math.sin(this.angle + 0.55 * Math.PI);
    this.air.x = 200 - 190 * Math.cos(this.angle + Math.PI);
    this.air.y = 205 - 190 * Math.sin(this.angle + Math.PI);
    this.water.x = 210 - 200 * Math.cos(this.angle - 0.45 * Math.PI);
    this.water.y = 225 - 215 * Math.sin(this.angle - 0.45 * Math.PI);

    this.pointed = null;
    for (var p in this.planets) {
      var planet = this.planets[p];
      if (planet.model && planet.model.freezes()) {
        planet.frame = FROZEN_FRAME;
      } else if (planet.model && planet.model.boils()) {
        planet.frame = BOILING_FRAME;
      } else {
        planet.frame = NORMAL_FRAME;
      }
      if (planet.input.pointerOver() && planet !== this.selected) {
        this.cursor.x = planet.x - planet.deltaCursor;
        this.cursor.y = planet.y - planet.deltaCursor;
        this.pointed = planet;
        break;
      }
    }
    if (!this.pointed) {
      this.cursor.x = -1000;
      this.cursor.y = -1000;
    }
    for (var p in this.planets) {
      var planet = this.planets[p];
      if (planet.input.justPressed()) {
        this.selectPlanet(planet, true);
        break;
      }
    }
    if (this.selected) {
      this.selectPlanet(this.selected);
      this.selection.x = this.selected.x - this.selected.deltaCursor;
      this.selection.y = this.selected.y - this.selected.deltaCursor;
    }

    if (this.textarea.input.pointerDown() && this.textnum == 0) {
      this.showText("Now do something!");
    }

    this.moveMoves();

    if (U.isAlright()) {
      game.state.start("credits");
    }
  },

  inSelection: false,
  selectOp: "",

  selectPlanet: function(planet, newSelection) {
    if (newSelection && this.inSelection) {
      this.inSelection = false;
      if (planet === this.selected) {
        return;
      }
      //console.log(this.selectOp + " transfers from " + this.selected.model.name +
      //            " to " + planet.model.name);
      var srcWorld = this.selected.model;
      var destWorld = planet.model;
      if (this.selectOp == "plus") {
        this.addMove(this.selected, planet, "fireelem");
        srcWorld.addTransfer(Elem.FIRE, destWorld, 1);
      } else if (this.selectOp = "minus") {
        this.removeMove(this.selected, planet, "fireelem");
        srcWorld.addTransfer(Elem.FIRE, destWorld, -1);
      }
      return;
    }
 
    this.selected = planet;
    var world = planet.model;
    if (!this.inSelection) {
      var text = world.name + "\n " +
        "Temp: " + numToStr(world.temp) + "\n " +
        "Freeze <" + numToStr(world.freeze) + "\n " +
        "Boils >" + numToStr(world.boil) + "\n " +
        "\n ";

      text += "Productions\n ";
      if (world.freezes()) {
        text += "* None (frozen)\n ";
      } else {
        for (var p in world.prods) {
          var production = world.prods[p];
          text += "* " + numToStr(production.qpr) + " " + production.elem.name + "\n ";
        }
        text += "\n ";
      }
      
      if (world.transfers.length > 0) {
        text += "Transfers\n ";
        for (var t in world.transfers) {
          var transfer = world.transfers[t];
          text += "* " + transfer.qpr + " " + transfer.elem.name +
            " to " + transfer.dest.name + "\n ";
        }
      }
      this.showText(text);
    }

    if (newSelection) {
      if (this.buttonPlus) {
        this.buttonPlus.destroy();
        this.buttonPlus = null;
      }
      if (this.buttonMinus) {
        this.buttonMinus.destroy();
        this.buttonMinus = null;
      }
      if (!world.freezes() && world.hasProductionLeft()) {
        this.buttonPlus = game.add.button(620, 500, "buttons",
                                          function() {
                                            this.inSelection = true;
                                            this.selectOp = "plus";
                                            this.showText("Please select destination world to " +
                                                          "add transfer.");
                                          },
                                          this, 2, 3, 2, 3);
        this.buttonPlus.smoothed = false;
        this.buttonPlus.scale.set(4);
      }
      if (world.transfers.length > 0) {
        this.buttonMinus = game.add.button(700, 500, "buttons",
                                           function() {
                                             this.inSelection = true;
                                             this.selectOp = "minus";
                                             this.showText("Please select destination world to " +
                                                           "decrease transfer");
                                           },
                                           this, 0, 1, 0, 1);
        this.buttonMinus.smoothed = false;
        this.buttonMinus.scale.set(4);
      }
    }
  },

  moveMoves: function() {
    for (var i = this.moves.length - 1; i >= 0; --i) {
      var move = this.moves[i];
      var destx = move.dest.x - move.dest.deltaCursor + 76;
      var desty = move.dest.y - move.dest.deltaCursor + 76;
      var dx = destx - move.sprite.x;
      var dy = desty - move.sprite.y;
      var dl = Math.sqrt(dx * dx + dy * dy);
      if (dl < 4) {
        move.sprite.x = move.src.x - move.src.deltaCursor + 76;
        move.sprite.y = move.src.y - move.src.deltaCursor + 76;
        continue;
      }
      var ux = dx / dl;
      var uy = dy / dl;
      move.sprite.x += ux;
      move.sprite.y += uy;
    }
  },

  showText: function(text, offsetY) {
    this.textgroup.removeAll(true, true);
    var words = text.split(" ");
    var y = offsetY || 20;
    for (var i = 0, line = ""; i <= words.length; ++i) {
      var word = words[i];
      if ((i == words.length) || (line.length + 1 + word.length > 18) ||
          (line[line.length - 1] == "\n")) {
        game.add.bitmapText(615, y, "nokiafont", line, 20, this.textgroup);
        line = "";
        y += 22;
      }
      line += " " + word;
    }
  },
  
  restartGame: function() {
    game.state.start("menu");
  },
};

//
var creditsState = {};
creditsState.preload = function() {
  game.stage.backgroundColor = "#303030";
  game.load.image("credits", "assets/credits.png");
  game.load.audio("creditstune", ["assets/creditstune.ogg", "assets/creditstune.mp3"], true);
};

creditsState.create = function() {
  this.back = game.add.sprite(0, 0, "credits");
  this.back.smoothed = false;
  this.back.scale.set(4);
  this.stars = game.add.sprite(0, 0, "stars");
  this.stars2 = game.add.sprite(800, 0, "stars");
  this.creditstune = game.add.audio("creditstune", 0.7, true, true);
  this.creditstune.play('', 0, 0.4, true);

  //this.button = game.add.button(168, 368, "start", menuState.gotoMain, this,
  //                              0, 1, 0, 1);
  //this.button.smoothed = false;
  //this.button.scale.set(4);

  this.longText = game.add.bitmapText(150, 600, "nokiafont",
                                      "Congratulations! You put the worlds in balance!\n\n\n" +
                                      "This pitiful game, has been done for LudumDare 30\n" +
                                      "using the following tools and libraries:\n\n" +
                                      "- Phaser (http://phaser.io)\n" +
                                      "   Desktop and Mobile HTML5 Game Framework\n\n" +
                                      "- emacs (http://www.gnu.org/software/emacs)\n" +
                                      "   THE editor\n\n" +
                                      "- Chrome (https://www.google.com/chrome)\n\n" +
                                      "- The Gimp (http://www.gimp.org)\n" +
                                      "   The GNU Image Manipulation Program\n\n" +
                                      "- MilkyTracker (http://www.milkytracker.org)\n" +
                                      "   Multi-platform ChipTune Composition Tool\n\n" +
                                      "- Cygwin (https://www.cygwin.com)\n" +
                                      "   Unix emulation and shell tools for Windows\n\n" +
                                      "- Git (http://git-scm.com)\n" +
                                      "   Distributed Revision Control\n\n" +
                                      "- FileZilla (https://filezilla-project.org)\n" +
                                      "   The Free FTP Solution\n\n" +
                                      "- Firefox (https://www.mozilla.org/firefox)\n\n\n\n\n\n" +
                                      "Thanks, and good night!",
                                      20);
};

creditsState.update = function() {
  this.stars.x -= 1;
  this.stars2.x -= 1;
  if (this.stars.x + 800 < 0) {
    this.stars.x += 800;
    this.stars2.x += 800;
  }
  this.longText.y -= 0.5;

  if (this.longText.y < -800) {
    this.creditstune.stop();
    game.state.start("menu");
  }
};




///
game.state.add("menu", menuState);
game.state.add("main", mainState);
game.state.add("credits", creditsState);
game.state.start("menu");
